package com.jkcredit.zuul.error;

import com.alibaba.fastjson.JSON;
import com.jkcredit.common.model.ResultObject;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.exception.ZuulException;
import org.springframework.cloud.netflix.zuul.filters.support.FilterConstants;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.UUID;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Component
public class ErrorFilter extends ZuulFilter {
    @Override
    public String filterType() {
        return FilterConstants.ERROR_TYPE;
    }

    @Override
    public int filterOrder() {
        return -1;
    }

    @Override
    public boolean shouldFilter() {
        return true;
    }

    @Override
    public Object run(){
        Object e = RequestContext.getCurrentContext().getThrowable();
        if (e != null && e instanceof ZuulException) {
            RequestContext.getCurrentContext().remove("throwable");
        }
        String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date())
                + UUID.randomUUID().toString();
        ResultObject resultObject = new ResultObject();
        resultObject.setCode(500);
        resultObject.setMessage("failed");
        resultObject.setRequestId(requestId);
        resultObject.setResult(new ArrayList<>());
        RequestContext.getCurrentContext().setResponseBody(JSON.toJSONString(resultObject));
        return resultObject;
    }
}
