package com.jkcredit.zuul.post;

import com.alibaba.fastjson.JSON;
import com.jkcredit.common.model.ResultObject;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.exception.ZuulException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.netflix.zuul.filters.support.FilterConstants;
import org.springframework.stereotype.Component;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Component
@Slf4j
public class EnvelopedResponseFilter extends ZuulFilter {
    @Override
    public String filterType() {
        return FilterConstants.POST_TYPE;
    }

    @Override
    public int filterOrder() {
        return FilterConstants.SERVLET_30_WRAPPER_FILTER_ORDER + 2;
    }

    @Override
    public boolean shouldFilter() {
        RequestContext requestContext = RequestContext.getCurrentContext();
        String url = requestContext.getRequest().getRequestURI();
        return url.contains("/query");
    }

    @Override
    public Object run() throws ZuulException {
        try {
            String response = RequestContext.getCurrentContext().getResponseBody();
            ResultObject resultObject = JSON.toJavaObject(JSON.parseObject(response),
                    ResultObject.class);
            RequestContext.getCurrentContext().setResponseBody(JSON.toJSONString(resultObject));
            log.info("requestId:{}, response:{}", resultObject.getRequestId(), JSON.toJSONString(resultObject));
        } catch (Exception e) {
            log.error("EnvelopedResponseFilter:", e);
        }

        return null;
    }
}
