package com.jkcredit.zuul.post;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.jkcredit.common.constant.CommonConstant;
import com.jkcredit.common.model.CommonResponseObject;
import com.jkcredit.zuul.util.InstanceStatusUtil;
import com.netflix.discovery.DiscoveryClient;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.exception.ZuulException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.netflix.zuul.filters.support.FilterConstants;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.lang.reflect.Method;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Component
@Slf4j
public class SeverUsefulCheckFilter extends ZuulFilter {
    @Autowired
    StringRedisTemplate stringRedisTemplate;
    @Autowired
    InstanceStatusUtil instanceStatusUtil;

    @Override
    public String filterType() {
        return FilterConstants.POST_TYPE;
    }

    @Override
    public int filterOrder() {
        return FilterConstants.SERVLET_30_WRAPPER_FILTER_ORDER + 1;
    }

    @Override
    public boolean shouldFilter() {
        RequestContext requestContext = RequestContext.getCurrentContext();
        String url = requestContext.getRequest().getRequestURI();
        return url.contains("/query");
    }

    @Override
    public Object run() throws ZuulException {
        String response = "";
        try {
            response = IOUtils.toString(RequestContext.getCurrentContext().getResponseDataStream());
            RequestContext.getCurrentContext().setResponseBody(response);
        } catch (IOException ioe) {
            ioe.printStackTrace();
            log.error("SeverUsefulCheckFilter.ioe:", ioe);
        }
        CommonResponseObject commonResponseObject = JSON.toJavaObject(JSON.parseObject(response),
                CommonResponseObject.class);

        if (commonResponseObject.getCode().equals(CommonConstant.GET_RESULT_FAILED)
                && commonResponseObject.getNetWorkCode().equals(CommonConstant.NETWORK_DISABLED)) {
            log.info("getCookieFailed:{}", commonResponseObject.getHost());
            stringRedisTemplate.boundSetOps("DISABLED_HOST").add(commonResponseObject.getHost());
            String instanceId = "info-query" + ":" + commonResponseObject.getHost();
            instanceStatusUtil.updateInstanceStatus(instanceId, "OUT_OF_SERVICE");
            try {
                Method method = DiscoveryClient.class.getDeclaredMethod("refreshRegistry");
                method.setAccessible(true);
                method.invoke(SpringUtil.getBean(DiscoveryClient.class));
            } catch (Exception e) {
                e.printStackTrace();
                log.error("SeverUsefulCheckFilter.e:", e);
            }
        }
        return null;
    }
}
