package com.jkcredit.zuul.task;

import cn.hutool.extra.spring.SpringUtil;
import com.jkcredit.common.constant.CommonConstant;
import com.jkcredit.common.model.CommonResponseObject;
import com.jkcredit.zuul.util.InstanceStatusUtil;
import com.netflix.discovery.DiscoveryClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.lang.reflect.Method;
import java.util.Set;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Component
@Slf4j
public class CheckClientNetworkTask {
    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @Resource(name = "getRestTemplate")
    RestTemplate restTemplate;

    @Autowired
    InstanceStatusUtil instanceStatusUtil;

    @Scheduled(cron = "0 0/5 * * * ?", zone = "Asia/Shanghai")
    public void checkClientNetwork() {
        Set<String> disableHosts = stringRedisTemplate.boundSetOps("DISABLED_HOST").members();
        for (String host : disableHosts) {
            String url = "http://" + host + "/getNewCookie";
            log.info("checkClientNetwork.url:{}", url);
            ResponseEntity<CommonResponseObject> responseEntity = restTemplate
                    .postForEntity(url, null, CommonResponseObject.class);
            CommonResponseObject responseObject = responseEntity.getBody();
            if (responseObject != null
                    && responseObject.getCode().equals(CommonConstant.GET_RESULT_SUCCESS)) {
                stringRedisTemplate.boundSetOps("DISABLED_HOST").remove(host);
                String instanceId = "info-query" + ":" + host;
                instanceStatusUtil.updateInstanceStatus(instanceId, "UP");

                try {
                    Method method = DiscoveryClient.class.getDeclaredMethod("refreshRegistry");
                    method.setAccessible(true);
                    method.invoke(SpringUtil.getBean(DiscoveryClient.class));
                } catch (Exception e) {
                    log.error("checkClientNetwork:", e);
                    e.printStackTrace();
                }
            } else {
                log.info("getNewCookieFailed：{}", host);
            }
        }
    }
}

